part of jaguar_auth.session;

/// JWT config model used to issue new JWT tokens
class JwtConfig {
  /// Issuer of the token
  final String issuer;

  /// Audience in the token
  final List<String> audience;

  /// Maximum duration for which the token is valid
  final Duration maxAge;

  /// HS256 key
  final String hmacKey;

  const JwtConfig(this.hmacKey,
      {this.issuer,
      this.audience: const [],
      this.maxAge: const Duration(days: 1)});
}

/// Contains information to validate JWT token against
class JwtValidationConfig {
  /// Issuer
  final String issuer;

  /// Audience
  final String audience;

  const JwtValidationConfig({this.issuer, this.audience});
}

abstract class _BaseJwtSession {
  JwtConfig get config;

  JwtValidationConfig get validationConfig;

  JwtClaim get _claimSet;

  set _claimSet(JwtClaim v);

  Map<String, String> get _outValues;

  String get _subject;

  /// Validates the current session
  ///
  /// Must throw exception on validation failure
  Future<Null> validate() async {
    if (_claimSet == null) {
      throw new Response(null, statusCode: HttpStatus.UNAUTHORIZED);
    }

    try {
      _claimSet.validate(
        issuer: validationConfig.issuer,
        audience: validationConfig.audience,
      );
    } on JwtException catch (_) {
      throw new Response(null, statusCode: HttpStatus.UNAUTHORIZED);
    }
  }

  String _encode() {
    final claimSet = new JwtClaim(
        issuer: config.issuer,
        subject: _subject,
        audience: config.audience,
        maxAge: config.maxAge,
        payload: _outValues);
    final String token = issueJwtHS256(claimSet, config.hmacKey);
    return const Base64Codec.urlSafe().encode(token.codeUnits);
  }

  Map<String, dynamic> _decode(String data) {
    final token =
        new String.fromCharCodes(const Base64Codec.urlSafe().decode(data));
    _claimSet = verifyJwtHS256Signature(token, config.hmacKey);

    final ret = new Map<String, dynamic>.from(_claimSet.payload);
    ret['sub'] = _claimSet.subject;

    return _claimSet.toJson();
  }
}
