library jaguar_auth.session;

import 'dart:io';
import 'dart:async';
import 'dart:convert';
import 'dart:collection';
import 'package:jaguar/jaguar.dart';
import 'package:auth_header/auth_header.dart';
import 'package:jaguar_jwt/jaguar_jwt.dart';

part 'jwt_common.dart';
part 'session_jwt_cookie.dart';
part 'session_jwt_auth_header.dart';
part 'session_cookie.dart';

/// Session management interface
///
/// SessionManager is responsible for:
/// 1. Parsing session identifier & information from the HTTP request
/// 2. Writing session identifier & information to HTTP response
/// 3. Loading and storing session data to session store
abstract class SessionManager {
  /// Returns a incoming session value by given [key]
  String getInValue(String key);

  /// Get incoming session data
  UnmodifiableMapView<String, String> get inValues;

  /// Get outgoing session data
  UnmodifiableMapView<String, String> get outValues;

  /// Creates a new session
  FutureOr<Null> createSession(Map<String, String> values);

  /// Updates the current session
  Future<Null> updateSession(Map<String, String> values);

  /// Deletes the current session
  Future<Null> deleteSession();

  /// Parses session from the given [request]
  Future<Null> parseRequest(Request request);

  /// Writes session data to the Response ([resp])
  Future<Response> writeResponse(Request request, Response resp);

  /// Validates the current session
  ///
  /// Must throw exception on validation failure
  Future<Null> validate();
}

/// Interceptor to parse and update session to [HttpRequest]
///
/// It allows offers an instance of [SessionManager] as input to other interceptors
/// and routes
class SessionInterceptor extends Interceptor {
  /// Session manager to parse and update session on [HttpRequest]
  ///
  /// Offered as [Input] to other interceptors and routes
  final SessionManager sessionManager;

  SessionInterceptor(this.sessionManager);

  Future<SessionManager> pre(Context ctx) async {
    await sessionManager.parseRequest(ctx.req);
    return sessionManager;
  }

  Future<Response> post(Context ctx, Response resp) async {
    return await sessionManager.writeResponse(ctx.req, resp);
  }
}
